/*
 * nlsop gui code
 *
 * author: frank zirkelbach (frank.zirkelbach@physik.uni-augsburg.de)
 *
 * this program tries helping to understand the amorphous depuration
 * and recrystallization of SiCx while ion implantation at temperatures
 * below 400 degree celsius.
 * hopefully the program will simulate the stabilization of the
 * selforganizing lamella structure in the observed behaviour.
 *
 * refs: 
 *  - J. K. N. Lindner. Habil.Schrift, Universitaet Augsburg.
 *  - Maik Haeberlen. Diplomarbeit, Universitaet Augsburg.
 *
 * Copyright (C) 2004 Frank Zirkelbach
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include "nlsop.h"
#include "dfbapi.h"
#include "random.h"

#include "network.h"
#include "event.h"
#include "display.h"
#include "input.h"

#include "nlsop_general.h"

#define LOGFILE "nlsop_gui_logfile"
#define GOON 2

int usage(char *prog)
{
 puts("usage:");
 printf("%s -i <ip> -p <port> -l <logfile>\n",prog);
 return 1;
}

/*
 * gui internal functions
 */

int display_new_line(t_display *display,t_input *input,char *text) {

  int x,y;
  int ptr;
  int i;

  unsigned char *tmp;

  x=display->max_x-1;
  y=display->max_y-1;
 
  tmp=(unsigned char *)malloc(display->max_x);
  memset(tmp,0x20,display->max_x);

  display_line(display,0,0,x,0,'#');
  display_string(display,x/2-17,1,"nlsop gui (C) 2004 Frank Zirkelbach",35);
  display_line(display,0,2,x,2,'#');
  display_line(display,0,y-1,x,y-1,'#');
  display_string(display,0,y,"prompt: ",8);
  memcpy(tmp,input->content,input->c_count);
  display_string(display,8,y,tmp,x-9);

  if(text!=NULL) {
    ptr=3*display->max_x;
    for(i=0;i<display->max_y-6;i++) {
      memcpy(display->screen+ptr,display->screen+ptr+display->max_x,
             display->max_x);
      ptr+=display->max_x;
    }
    i=strlen(text);
    memcpy(display->screen+ptr,text,i);
    memset(display->screen+ptr+i,0x20,x-i);
  }

  display_draw(display);
  display_set_cursor(display,8+input->c_count,y);

  return 1;
}

int send_and_wait_for_answer(t_net *net,t_display *display,t_input *input) {

  int i,count;
  t_job job;
  char data;
  char string[MAX_CONTENT];

  data=GUI_INFO;

  /* request information */
  network_send_chan(net,0,&data,sizeof(unsigned char));

  /* receive information */
  network_receive_chan(net,0,&data,sizeof(unsigned char));
  if(data!=GUI_INFO) {
    display_new_line(display,input,"no gui info answer, ignored");
    return -1;
  }

  network_receive_chan(net,0,(unsigned char *)&count,sizeof(int));
  for(i=0;i<count;i++) {
    network_receive_chan(net,0,(unsigned char *)&job,sizeof(t_job));
    snprintf(string,MAX_CONTENT,
             "channel %d %c => b=%f c=%f s=%f | d: %d %f | %d",
             job.channel,job.status&IN_WORK?'a':'q',
             job.info.b,job.info.c,job.info.s,
             job.info.diff_rate,job.info.dr_ac,job.progress);
    display_new_line(display,input,string);
  }

  display_new_line(display,input,"done");

  return 1;
}

int send_job(t_net *net,t_input *input,t_display *display) {

  char command[MAX_CONTENT];
  int x,y,z;
  info info;

  /* default values */
  x=_X;
  y=_Y;
  z=_Z;
  info.cc=CC;
  info.steps=STEPS;
  info.range=RANGE;
  info.diff_rate=DIFF_RATE;
  info.cpi=CPI;
  info.s_rate=S_RATE;
  info.save_rate=RESAVE;
  info.s=S_D;
  info.b=B_D;
  info.c=C_D;
  info.dr_ac=DR_AC;

  strncpy(command,input->content,MAX_CONTENT);

  strtok(command," ");
  while(strtok(NULL," ")!=NULL) {
    switch(command[0]) {
      case 'b':
        info.b=atof(command+1);
        break;
      case 'c':
        info.c=atof(command+1);
        break;
      case 's':
        info.s=atof(command+1);
        break;
      case 'd':
        info.diff_rate=atoi(command+1);
        break;
      case 'D':
        info.dr_ac=atof(command+1);
        break;
      case 'S':
        info.steps=atoi(command+1);
        break;
      case 'C':
        info.cpi=atoi(command+1);
        break;
      case 'r':
        info.range=atoi(command+1);
        break;
      case 'R':
        info.s_rate=atoi(command+1);
        break;
      case 'x':
        info.save_rate=atoi(command+1);
        break;
      case 'X':
        x=atoi(command+1);
        break;
      case 'Y':
        y=atoi(command+1);
        break;
      case 'Z':
        z=atoi(command+1);
        break;
      default:
        display_new_line(display,input,"unknown command");
        break;
    }
  }

  command[0]=GUI_ADDJOB;
  network_send_chan(net,0,command,1);

  network_send_chan(net,0,(unsigned char *)&x,sizeof(int));
  network_send_chan(net,0,(unsigned char *)&y,sizeof(int));
  network_send_chan(net,0,(unsigned char *)&z,sizeof(int));

  network_send_chan(net,0,(unsigned char *)&info,sizeof(info));

  return 1;
}

int get_whatever(t_input *input,void *ptr) {

  char last;
  t_display *display;

  display=(t_display *)ptr;

  last=input->content[input->c_count-1];

  if((last=='\n')||(last=='\r')) {
    input->content[input->c_count-1]='\0';
    input->c_count=0;
    return GOON;
  }

  display_new_line(display,input,NULL);

  return 1;
}

int nothing(t_event *event,void *allineed) {

  return 1;
}

int send_quit(t_net *net) {

  unsigned char data;

  data=GUI_QUIT;

  network_send_chan(net,0,&data,1);

  return 1;
}

int get_command(t_event *event,void *allineed) {

  t_net *net;
  t_display *display;
  t_input *input;
  unsigned int addr[3];

  memcpy(addr,allineed,3*sizeof(unsigned int));
  net=(t_net *)addr[0];
  display=(t_display *)addr[1];
  input=(t_input *)addr[2];

  input_get_event(input,get_whatever,display);

  /* if there was a new line! */
  if(input->c_count==0) {
    switch(input->content[0]) {
      case GUI_INFO:
        send_and_wait_for_answer(net,display,input);
        break;
      case GUI_ADDJOB:
        send_job(net,input,display);
        break;
      case GUI_QUIT:
        /* stop everything */
        event_math(0,event,READ,REMOVE);
        send_quit(net);
        network_shutdown(net);
        input_shutdown(input);
        display_shutdown(display);
        event_stop(event);
        return 2;
      default:
        display_new_line(display,input,"unknown gui command");
        break;
    }
  }

  return 1;
}

/*
 * main program
 */

int main(int argc,char **argv)
{

  char server_ip[16];
  char logfile[64];
  int port;

  t_net net;
  t_event event;
  t_display display;
  t_input input;

  unsigned char data;
  int i;

  int fd;

  unsigned int addr[3];
  void *allyouneed;

  allyouneed=(void *)addr;

  addr[0]=(unsigned int)&net;
  addr[1]=(unsigned int)&display;
  addr[2]=(unsigned int)&input;

  /* default values */
  strcpy(logfile,LOGFILE);
  strcpy(server_ip,"");
  port=1025;

  /* parse/check argv */
  for(i=1;i<argc;i++) {
    if(argv[i][0]=='-') {
      switch(argv[i][1]) {
        case 'h':
          usage(argv[0]);
          return -1;
        case 'i':
          strncpy(server_ip,argv[++i],16);
          break;
        case 'p':
          port=atoi(argv[++i]);
          break;
        case 'l':
          strncpy(logfile,argv[++i],64);
          break;
        default:
          usage(argv[0]);
          return -1;
      }
    }
  }
  if(!strcmp(server_ip,"")) {
    usage(argv[0]);
    return -1;
  }

  if((fd=open(logfile,O_WRONLY|O_CREAT))<0) {
    printf("unable to open file %s\n",logfile);
    return -1;
  }

  /* display init */
  display_init(&display,fd);

  /* input init */
  input_init(&input,fd);
  input.mode=CONTENT_BUFFER|ECHO;
  input_ios_init(&input);

  /* event init */
  event_init(&event,fd);
  event_set_timeout(&event,0,0);

  /* user interaction */
  event_math(0,&event,READ,ADD);

  display_new_line(&display,&input,"welcome to nlsop gui! :)");

  /* connect to server */
  network_init(&net,fd);
  network_set_connection_info(&net,0,server_ip,port);
  if(network_connect(&net,0)==N_E_CONNECT) {
    printf("unable to connect to server, aborting ...\n");
    return -1;
  }
  network_select(&net,0);

  display_new_line(&display,&input,"successfully connected to server");

  /* tell server: i am a client, i may work for you */
  data=NLSOP_GUI;
  network_send(net.connection[0].fd,&data,1);

  display_new_line(&display,&input,"send data to server, waiting for input");

  /* wait for job */
  event_start(&event,allyouneed,get_command,nothing);

  close(fd);

  return 1;
}
